;------------------------------------------
; Usage:
; SYNCHRO_TIMESTAMP = $02c0
; ldx #<SYNCHRO_TIMESTAMP
; ldy #>SYNCHRO_TIMESTAMP
; jsr synchro_check
;------------------------------------------
; Synchronization check will loop until
; .X/.Y is greater than SYNCHRO_COUNTER
;------------------------------------------
_synchro_check  ; The following condition is verified here until it is true:
                ; while (.X/.Y > SYNCHRO_COUNTER+0/SYNCHRO_COUNTER+1) {}
                ; Begin with a delaying loop, no need to check it continuously:
                jsr _raster_wait
                ; Store SYNCHRO_COUNTER into synchro_compare variable to avoid
                ; blocking IRQ routine for an entire time of calculation check:
                sei
                lda SYNCHRO_COUNTER + $00
                sta synchro_compare + $00
                lda SYNCHRO_COUNTER + $01
                sta synchro_compare + $01
                cli
                ; If .Y == synchro_compare+1, perform additional check with .X:
                cpy synchro_compare + 1
                beq *+12
                ; If .Y < synchro_compare+1, we are done:
                tya
                sec
                sbc synchro_compare + 1
                bcc *+20
                ; If .Y > synchro_compare+1, we still need to wait:
                jmp _synchro_check
                ; If .X == synchro_compare+0, we are done:
                cpx synchro_compare + 0
                beq *+12
                ; If .X < synchro_compare+0, we are done:
                txa
                sec
                sbc synchro_compare + 0
                bcc *+5
                ; If .X > synchro_compare+0, we still need to wait:
                jmp _synchro_check
                rts
;------------------------------------------
synchro_compare .db $00,$00
;------------------------------------------
; Usage:
; SYNCHRO_TIMESTAMP = $02c0
; ldx #<SYNCHRO_TIMESTAMP
; ldy #>SYNCHRO_TIMESTAMP
; jsr synchro_check_no_wait
;------------------------------------------
; Synchronization check will:
;  * set carry when .X/.Y is greater than SYNCHRO_COUNTER (synchronisation counter has been reached)
;  * clear carry when .X/.Y is less than SYNCHRO_COUNTER (synchronisation counter has not yet been reached)
;------------------------------------------
_synchro_check_no_wait
                ; This function is dedicated for use inside an IRQ routine, thus
                ; there is no need to store SYNCHRO_COUNTER into a variable,
                ; because blocking an IRQ routine won't ever be the case here:
                ; sei
                ; lda SYNCHRO_COUNTER + $00
                ; sta synchro_compare + $00
                ; lda SYNCHRO_COUNTER + $01
                ; sta synchro_compare + $01
                ; cli
                ; The following condition is verified here whether it is true:
                ; if (.X/.Y > SYNCHRO_COUNTER+0/SYNCHRO_COUNTER+1) { sec } else { clc }
                ; If .Y == SYNCHRO_COUNTER+1, perform additional check with .X:
                cpy SYNCHRO_COUNTER + 1
                beq *+10
                ; If .Y < SYNCHRO_COUNTER+1, counter has not yet been reached:
                tya
                sec
                sbc SYNCHRO_COUNTER + 1
                bcc *+16
                ; If .Y > SYNCHRO_COUNTER+1, counter has not yet been reached:
                clc
                rts
                ; If .X == SYNCHRO_COUNTER+0, counter has been reached:
                cpx SYNCHRO_COUNTER + 0
                beq *+10
                ; If .X < SYNCHRO_COUNTER+0, counter has been reached:
                txa
                sec
                sbc SYNCHRO_COUNTER + 0
                bcc *+4
                ; If .X > SYNCHRO_COUNTER+0, counter has not yet been reached:
                clc
                rts
                sec
                rts
;------------------------------------------
