package Helper;

use utf8;
use bytes;
use strict;
use warnings;

use base qw(Exporter);
our %EXPORT_TAGS = ();
$EXPORT_TAGS{graphics} = [ qw(&build_picture_from_fonts_screen_data) ];
$EXPORT_TAGS{source} = [ qw(&get_data_as_source_code print_data_as_source_code save_text_file) ];
$EXPORT_TAGS{all} = [ @{ $EXPORT_TAGS{source} }, @{ $EXPORT_TAGS{graphics} } ];
our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
our @EXPORT = qw();

use Data::Dumper;
use Graphics::Factory;
use IO::File;

sub get_data_as_source_code {
  my ($data, $label) = @_;

  my $indent = 16;

  my $source_code;

  if ($label) {
    $source_code .= get_source_code_comment($indent, $data);
    $source_code .= ' ' x $indent . "*= $label\n";
  }
  $source_code .= get_source_code_comment($indent, $data);

  for my $row (@{$data}) {
    $source_code .= ' ' x $indent . '.db ';
    $source_code .= join ',', map { sprintf '$%02x', $_ } @{$row};
    $source_code .= "\n";
  }

  $source_code .= get_source_code_comment($indent, $data);

  return $source_code;
}

sub print_data_as_source_code {
  my ($data, $label) = @_;

  my $source_code = get_data_as_source_code($data, $label);

  print $source_code;

  return;
}

sub get_source_code_comment {
  my ($indent, $data) = @_;

  my $num_row_values = scalar @{$data->[0]};
  my $data_length = $num_row_values * 3 + ($num_row_values - 1);

  my $comment_length = 4 + $data_length + $indent - 1;

  my $source_code_comment = sprintf ";%s\n", '-' x $comment_length;

  return $source_code_comment;
}

sub print_source_code_comment {
  my ($indent, $data) = @_;

  my $source_code_comment = get_source_code_comment($indent, $data);

  printf $source_code_comment;

  return;
}

sub build_picture_from_fonts_screen_data {
  my ($charset, $screen, $size) = @_;

  # 1. Build C64 bitmap data from charset/screen:
  my $bytes;
  for my $char_index (@{$screen}) {
    my $from = $char_index * 0x08;
    my $to = $from + 0x08;
    for (my $byte_index = $from; $byte_index < $to; $byte_index++) {
      my $char_pixel_row_data = $charset->[$byte_index];
      $bytes .= chr $char_pixel_row_data;
    }
  }

  # 2. Build hires picture from raw data bytes:
  my $picture = Graphics::Factory->hires(data => $bytes, dimensions => $size);

  return $picture;
}

sub save_text_file {
  my ($file, $data) = @_;

  my $fh = IO::File->new($file, 'w');
  die "Unable to write file: $file" unless defined $fh;

  $fh->binmode(':utf8');
  print $fh $data;

  $fh->close;
  return;
}

1;

__END__