/** How to execute this script:
  *
  *   $ export AFTERIMAGE=/home/pkrol/.ivy2/cache/org.c64.attitude/afterimage/jars/afterimage-0.03-SNAPSHOT.jar
  *   $ export IMAGEJ=/home/pkrol/.ivy2/cache/gov.nih.imagej/imagej/jars/imagej-1.46.jar
  *   $ scala -classpath ${AFTERIMAGE}:${IMAGEJ} scripts/convert-image-to-c64-palette.scala images/question-mark-02-resized.png images/question-mark-03-converted.png
  */

import org.c64.attitude.Afterimage.Colour.Palette
import org.c64.attitude.Afterimage.File.File
import org.c64.attitude.Afterimage.File.Import
import org.c64.attitude.Afterimage.File.Type.PNG
import org.c64.attitude.Afterimage.Format.FacePainter
import org.c64.attitude.Afterimage.Mode
import org.c64.attitude.Afterimage.View.Image

import scala.sys.process._

def usage() {
  println("  Usage: scripts/convert-image-to-c64-palette.scala <SOURCE.PNG> <TARGET.PNG>")
}

val source = if (args.length > 0) args(0) else ""
val target = if (args.length > 1) args(1) else ""

if (source == "") {
  println("Source PNG file must be specified")
  usage()
  exit(1)
}

if (!new java.io.File(source).exists) {
  println("Source PNG file does not exist: \"%s\"".format(source))
  usage()
  exit(2)
}

if (target == "") {
  println("Target PNG file must be specified")
  usage()
  exit(3)
}

if (new java.io.File(target).exists) {
  println("Target PNG file already exists: \"%s\"".format(target))
  usage()
  exit(4)
}

val palette = Palette("default")

println("Importing source image file: \"%s\"".format(source))

val importMode = Import.MultiColour(backgroundColour = 0x01, palette)
val picture = File.convert(source, importMode)
val facePainter = FacePainter(picture.asInstanceOf[Mode.MultiColour])

println("Writing target image file: \"%s\"".format(target))

val cbmData = facePainter.extractData()
val preview = Image(cbmData, palette)
// preview.show()
val png = PNG(preview)
png.save(target)
"convert -resize 320x200 %s %s".format(target, target).!
