#!/usr/bin/perl
#
# This script rebuilds PNG images out of a source code charset and screen data.
# Making "setup-soci-suca.pl" script able to save those pictures makes this
# script actually obsolete.
#
# scripts/rebuild-soci-suca-pictures.pl soci-wjazd-screen-and-charset.inc soci.png suca.png

use utf8;
use bytes;
use strict;
use warnings;

use FindBin qw($Bin);
use lib $Bin;

use Data::Dumper;
use Helper qw(:graphics);
use IO::File;

my ($input, $soci_file, $suca_file) = @ARGV;

my $data = read_input_data($input);

my $soci_image = build_picture_from_data($data, 'SOCI');
my $suca_image = build_picture_from_data($data, 'SUCA');
save_picture_to_file($soci_image, $soci_file);
save_picture_to_file($suca_image, $suca_file);

sub save_picture_to_file {
  my ($image, $file) = @_;
  die "File already exists: $file" if -e $file;
  $image->save_preview(to_file => $file);
  return;
}

sub build_picture_from_data {
  my ($data, $which_prefix) = @_;
  my $charset = $data->{"${which_prefix}_WJAZD_CHARSET_DATA"};
  my $screen = $data->{"${which_prefix}_WJAZD_SCREEN_DATA"};
  my $size = sprintf '40x%d', scalar (@{$screen}) / 40;
  my $picture = build_picture_from_fonts_screen_data($charset, $screen, $size);
  return $picture;
}

sub read_input_data {
  my ($input) = @_;
  my $fh = new IO::File $input, 'r';
  die "No such file: $input" unless defined $fh;
  local $/;
  my $content = <$fh>;
  $fh->close;
  my %data;
  for my $section (qw(SOCI_WJAZD_SCREEN_DATA SOCI_WJAZD_CHARSET_DATA SUCA_WJAZD_SCREEN_DATA SUCA_WJAZD_CHARSET_DATA)) {
    $data{$section} = read_section_data($section, $content);
  }
  return \%data;
}

sub read_section_data {
  my ($section, $data) = @_;
  my ($data_bytes) = $data =~ m/\*= $section\n;-+\n(.*?);-+\n/s;
  my @section_data;
  while ($data_bytes =~ m/\$([0-9a-f]{2})/ig) {
    push @section_data, hex $+;
  }
  return \@section_data;
}
