#!/usr/bin/perl
#
# scripts/setup-2014.pl --target-data=target/sprites-2014-data.inc --source-image=images/2014.png

use utf8;
use bytes;
use strict;
use warnings;

local $| = 1;

use lib q{../../tools/lib};

use Data::Dumper;
use Getopt::Long;
use Graphics::Builder;
use Graphics::HiRes;
use Graphics::Transform qw(:all);

use FindBin qw($Bin);
use lib $Bin;
use Helper qw(:all);

# Get options:
my %options = get_options();

# Read PNG image:
my $image = read_source_image($options{source_image});

# Convert image to sprites:
my $sprites = convert_image_to_sprites($image);

# Save sprite data to target data file:
save_sprite_data_to_file($sprites, $options{target_data});

sub save_sprite_data_to_file {
  my ($data, $file) = @_;

  print qq{Saving generated source code data to "$file" file... };

  my $source_code = get_data_as_source_code($data, 'SPRITES_2014_DATA');

  save_text_file($file, $source_code);

  print "OK!\n";

  return;
}

sub convert_image_to_sprites {
  my ($image) = @_;

  print qq{Converting hires image data to sprites... };

  my $data = $image->get_raw_data(rows => [ 0 .. 5 ]);

  my $sprites = convert_plain_bitmap_to_sprites(
    data => $data,
    rows => 2,
    cols => 3,
  );

  print "OK!\n";

  # warn Dumper [ map { join '', map { unpack 'H*', chr $_ } @{$_} } @{$sprites} ];

  return $sprites;
}

sub read_source_image {
  my ($filename) = @_;

  print qq{Building hires picture from "$filename" file... };

  my $image = Graphics::Builder->new_from(png_file => $filename);

  my $bytes = $image->to_hires_bitmap();
  my $picture = Graphics::HiRes->new(data => $bytes);

  print "OK!\n";

  # $picture->preview();

  return $picture;
}

sub get_options {
  my %options = (
    source_image => undef,
    target_data  => undef,
  );

  GetOptions("source-image=s" => \$options{source_image},
             "target-data=s"  => \$options{target_data})
  or die("Error in command line arguments\n");

  die "Missing argument: --source-image=<FILENAME>" unless defined $options{source_image};
  die "File does not exists: $options{source_image}" unless -e $options{source_image};

  die "Missing argument: --target-data=<FILENAME>" unless defined $options{target_data};
  die "File already exists: $options{target_data}" if -e $options{target_data};

  return %options;
}
