#!/usr/bin/perl
#
# scripts/setup-horna.pl --data=target/horna-wjazd-screen-and-charset.inc --horna=target/horna.png

use utf8;
use bytes;
use strict;
use warnings;

local $| = 1;

use lib q{../../tools/lib};

use Data::Dumper;
use Data::Transform qw(:all);
use Getopt::Long;
use Graphics::Builder;
use Graphics::HiRes;
use Graphics::Transform qw(:all);

use FindBin qw($Bin);
use lib $Bin;
use Helper qw(:all);

# Get options:
my %options = get_options();

# Setup "HORNA" text:
my ($fonts_data, $screen_map) = build_horna_text_and_convert_it_to_fonts($options{horna});

## $fonts_data -> contains 35 fonts data ($81..$a3)
## $screen_map -> contains 3 rows of screen data

# Setup source code for "HORNA" screen map and charset:
my $soci_screen_map_source_code = get_generated_screen_map_and_charset_as_source_code($fonts_data, $screen_map);

# Write generated "HORNA" source code data into target file:
save_text_file($options{data}, $soci_screen_map_source_code);

sub get_generated_screen_map_and_charset_as_source_code {
  my ($fonts_data, $screen_map) = @_;

  my $source_code = get_data_as_source_code($screen_map, 'HORNA_WJAZD_SCREEN_DATA');

  $source_code .= "\n";

  $source_code .= get_data_as_source_code($fonts_data, 'HORNA_WJAZD_CHARSET_DATA');

  return $source_code;
}

sub build_horna_text_and_convert_it_to_fonts {
  my ($target_pic_file) = @_;

  my $text = 'HORNA';

  print "Building graphic image of $text... ";

  my $image = Graphics::Builder->new();
  $image->write_text(size => 26, text => $text, font => 'DejaVuSansMono-Bold');

  my $bytes = $image->to_hires_bitmap();
  my $picture = Graphics::HiRes->new(data => $bytes);

  print "OK!\n";

  # $picture->preview();

  my $data = $picture->get_non_empty_raw_data();

  my $cols = 40;
  my $rows = length ($data) / 8 / $cols;

  print "Saving image to file $target_pic_file... ";

  $picture->save_preview(to_file => $target_pic_file);

  print "OK!\n";

  my ($fonts, $screen_map) = convert_plain_bitmap_to_fonts(
    data   => $data,
    rows   => $rows,
    cols   => $cols,
  );

  my @screen_map_data = map {
    [ convert_binary_data_to_hex_values(data => $_) ]
  } @{ $screen_map };

  my @fonts_data = map {
    [ convert_binary_data_to_hex_values(data => $_) ]
  } @{ $fonts };

  # warn Dumper { screen_map => [ map { join '', map { unpack 'H2', chr } @$_ } @screen_map_data ] };
  # warn Dumper { fonts => [ map { join '', map { unpack 'H2', chr } @$_ } @fonts_data ] };

  return (\@fonts_data, \@screen_map_data);
}

sub get_options {
  my %options = (
    data  => undef,
    horna => undef,
  );

  GetOptions ("data=s"  => \$options{data},
              "horna=s" => \$options{horna})
  or die("Error in command line arguments\n");

  while (my ($param, $value) = each %options) {
    die "Missing argument: --$param=<FILENAME>" unless defined $value;
    die "File already exists: $value" if -e $value;
  }

  return %options;
}
