#!/usr/bin/perl
#
# scripts/setup-welcome-to.pl

use utf8;
use bytes;
use strict;
use warnings;

local $| = 1;

use lib q{../../tools/lib};

use Data::Dumper;
use Data::Transform qw(:all);
use Graphics::Builder;
use Graphics::HiRes;
use Graphics::Transform qw(:all);
use Readonly;

use FindBin qw($Bin);
use lib $Bin;
use Helper qw(:all);

Readonly my $NUM_SPACES_BETWEEN_WELCOME_AND_TO => 2;

my ($fonts, $welcome_screen_map) = build_welcome_text_and_convert_it_to_fonts();
my ($fonts_data, $to_screen_map) = build_to_text_and_convert_it_to_fonts($fonts);

## $welcome_screen_map -> contains 5 rows of screen data
## $to_screen_map      -> contains 4 rows of screen data
## $fonts_data         -> contains 128 fonts data ($00..$7f)

print_generated_data_as_source_code($welcome_screen_map, $to_screen_map, $fonts_data);

sub print_generated_data_as_source_code {
  my ($welcome_screen_map, $to_screen_map, $fonts_data) = @_;

  my @spacing_between_welcome_and_to;
  for ( 1 .. $NUM_SPACES_BETWEEN_WELCOME_AND_TO ) {
    push @spacing_between_welcome_and_to, [ map { 0x00 } ( 0x00 .. 0x27 ) ];
  }

  my $screen_map = [ @{$welcome_screen_map}, @spacing_between_welcome_and_to, @{$to_screen_map} ];

  print_data_as_source_code($screen_map, 'WELCOME_TO_SCREEN_DATA');

  print "\n";

  print_data_as_source_code($fonts_data, 'WELCOME_TO_CHARSET_DATA');

  return;
}

sub build_welcome_text_and_convert_it_to_fonts {

  my $image = Graphics::Builder->new();
  $image->write_text(size => 48, text => 'Welcome');

  my $bytes = $image->to_hires_bitmap();
  my $picture = Graphics::HiRes->new(data => $bytes);

  # $picture->preview();

  my $data = $picture->get_non_empty_raw_data();

  my $cols = 40;
  my $rows = length ($data) / 8 / $cols;

  # $picture->save_preview(to_file => 'welcome.png');

  my ($fonts, $screen_map) = convert_plain_bitmap_to_fonts(
    data => $data,
    rows => $rows,
    cols => $cols,
  );

  my @screen_map_data = map {
    [ convert_binary_data_to_hex_values(data => $_) ]
  } @{ $screen_map };

  # warn Dumper { screen_map => [ map { join '', map { unpack 'H2', chr } @$_ } @screen_map_data ] };

  return ($fonts, \@screen_map_data);
}

sub build_to_text_and_convert_it_to_fonts {
  my ($charset) = @_;

  my $image = Graphics::Builder->new();
  $image->write_text(size => 36, text => 'to');

  my $bytes = $image->to_hires_bitmap();
  my $picture = Graphics::HiRes->new(data => $bytes);

  # $picture->preview();

  my $data = $picture->get_non_empty_raw_data();

  my $cols = 40;
  my $rows = length ($data) / 8 / $cols;

  # $picture->save_preview(to_file => 'to.png');

  my ($fonts, $screen_map) = convert_plain_bitmap_to_fonts(
    data => $data,
    rows => $rows,
    cols => $cols,
    reusable_fonts => $charset,
  );

  my @screen_map_data = map {
    [ convert_binary_data_to_hex_values(data => $_) ]
  } @{ $screen_map };

  my @fonts_data = map {
    [ convert_binary_data_to_hex_values(data => $_) ]
  } @{ $fonts };

  # warn Dumper { screen_map => [ map { join '', map { unpack 'H2', chr } @$_ } @screen_map_data ] };
  # warn Dumper { fonts => [ map { join '', map { unpack 'H2', chr } @$_ } @fonts_data ] };

  return (\@fonts_data, \@screen_map_data);
}
