#!/usr/bin/perl
#
# scripts/setup-dream-logo.pl

use utf8;
use bytes;
use strict;
use warnings;

local $| = 1;

use lib q{../../tools/lib};

use Data::Dumper;
use Data::Transform qw(:all);
use FindBin qw($Bin);
use Graphics::KoalaPainter;
use Readonly;

use lib $Bin;
use Helper qw(:all);

Readonly our $logo => sprintf '%s/../images/PIC_LOGO_NR2.kla', $Bin;
Readonly our $logo_width => 10;

my $picture = Graphics::KoalaPainter->new(file => $logo);
my $bitmap = $picture->_get_bitmap_data();
my $screen = $picture->_get_screen_data();
my $colors = $picture->_get_colors_data();

my $data = {};

while ($bitmap) {
  $data->{bitmap} .= substr $bitmap, 0x0000, $logo_width * 0x08, '';
  substr $bitmap, 0x0000, 0x0140 - $logo_width * 0x08, '';
}

while ($screen) {
  $data->{screen} .= substr $screen, 0x0000, $logo_width, '';
  substr $screen, 0x0000, 0x0028 - $logo_width, '';
}

while ($colors) {
  $data->{colors} .= substr $colors, 0x0000, $logo_width, '';
  substr $colors, 0x0000, 0x0028 - $logo_width, '';
}

my %data;

for my $component (qw(bitmap screen colors)) {
  my @bytes = convert_binary_data_to_hex_values(data => $data->{$component});
  while (@bytes) {
    my @chunk = splice @bytes, 0x00, 0x08;
    push @{$data{$component}}, \@chunk;
  }
}

# $data{bitmap} - 2000 bytes of 10x25 chars data (row by row)
# $data{screen} - 250 bytes of 10x25 chars data (row by row)
# $data{colors} - 250 bytes of 10x25 chars data (row by row)

my $comment = ";--------------------------------------------------\n";

my $source_code = $comment;
$source_code .= "ALIAS_MEDRON_NUM_ROWS = 25\n";
$source_code .= "ALIAS_MEDRON_NUM_COLS = $logo_width\n";
$source_code .= $comment;
$source_code .= "\n";
$source_code .= get_data_as_source_code($data{bitmap}, 'ALIAS_MEDRON_BITMAP');
$source_code .= "\n";
$source_code .= get_data_as_source_code($data{screen}, 'ALIAS_MEDRON_SCREEN');
$source_code .= "\n";
$source_code .= get_data_as_source_code($data{colors}, 'ALIAS_MEDRON_COLORS');

print $source_code;
