#!/usr/bin/perl
#
# Dump source data:
#
# perl vector2.pl
#
# Animate prototype:
#
# perl vector2.pl --animate

use utf8;
use bytes;
use strict;
use warnings;

use Data::Dumper;
use Getopt::Long;
use Readonly;
use Tk;

Readonly our $colour1 => '#aa0000';
Readonly our $colour2 => '#228822';
Readonly our $colour3 => '#0000aa';

Readonly our $width  => 200;
Readonly our $height => 200;
Readonly our $bckgrd => '#111111';

use lib q{scripts};
use Vector3D::Plane::Easy;

use FindBin qw($Bin);
use lib $Bin;
use Vector1;

# Get options:
my %options = get_options();

my $vector = Vector1->new(width => $width, height => $height, points => setup_points(), polygons => setup_polygons(), colours => [$colour1, $colour2, $colour3]);

my $source_code = $vector->get_lines_source_code();
print $source_code;

# Animate prototype when "animate" option has been specified:
if ($options{animate}) {

  my $mw = MainWindow->new;

  my $canvas = $mw->Canvas(-width => 2 * $width, -height => 2 * $height, -background => $bckgrd)->pack;

  $mw->after(1 => sub { do_draw($vector, $mw, $canvas) });

  MainLoop;
}

sub do_draw {
  my ($vector, $mw, $canvas) = @_;
  $canvas->delete('all');

  my $outline = '#000000';

  for my $plane (@{$vector->next()}) {

    my $scale = 100 / $Vector1::scale;
    my $translate_x = $width - 32;
    my $translate_y = $height - 32;

    my @vertices = map { $_->[0], $_->[1] } $plane->get_vertices();

    my $colour  = $plane->get_colour();

    $canvas->createPolygon(@vertices, -fill => $colour, -outline => $outline, -width => 1);
  }

  ## Next frame
  $mw->after($vector->speed() => sub { do_draw($vector, $mw, $canvas) });
}

sub get_options {
  my %options = (
    animate => 0,
  );

  GetOptions ("animate" => \$options{animate})
  or die("Error in command line arguments\n");

  return %options;
}

sub setup_points {

  my @points = (
    [ +1, -1, -1 ],
    [ -1, -1, -1 ],
    [ -1, -1, +1 ],
    [ +1, -1, +1 ],
    [ +0, +1, +0 ],
  );

  return \@points;
}

sub setup_polygons {

  my @polygons = (
    {
      colour   => $colour1,
      vertices => [ 3, 2, 1, 0 ],
    },
    {
      colour   => $colour2,
      vertices => [ 2, 3, 4 ],
    },
    {
      colour   => $colour3,
      vertices => [ 1, 2, 4 ],
    },
    {
      colour   => $colour2,
      vertices => [ 0, 1, 4 ],
    },
    {
      colour   => $colour3,
      vertices => [ 3, 0, 4 ],
    },
  );

  return \@polygons;
}

__END__
