#!/usr/bin/perl
#
# scripts/calculate-vector-scroll-sinus.pl

use utf8;
use bytes;
use strict;
use warnings;

local $| = 1;

use lib q{../../tools/lib};

use Data::Dumper;
# use Data::Transform qw(:all);
use Readonly;

use FindBin qw($Bin);
use lib $Bin;
use Helper qw(:all);

Readonly my $PI => 3.14;
Readonly my $NUM_VALUES => 0x40;
Readonly my $MAX_VALUE => 0x0140;
Readonly my $MIN_VALUE => 0x0080;

# Calculate sinus from $0140 to $00a0:
my $data = calculate_sinus_data();
# Split sinus data into two tables:
my ($screen_positions, $d016_offsets) = split_sinus_data($data);

# Print out generated lookup table:
my (@screen_positions_split, @d016_offsets_split);
while (@{$screen_positions}) {
  push @screen_positions_split, [ splice @{$screen_positions}, 0x00, 0x08 ];
  push @d016_offsets_split, [ splice @{$d016_offsets}, 0x00, 0x08 ];
}

print_data_as_source_code(\@screen_positions_split, 'VECTOR_SCROLL_POSITIONS');
print "\n";
print_data_as_source_code(\@d016_offsets_split, 'VECTOR_SCROLL_D016_OFFSETS');

sub calculate_sinus_data {
  my @data;
  for my $index (0x01 .. $NUM_VALUES) {
    my $sinus = int ((1 - sin ($PI / 2 * $index / $NUM_VALUES)) * ($MAX_VALUE - $MIN_VALUE)) + $MIN_VALUE;
    push @data, $sinus;
  }
  return \@data;
}

sub split_sinus_data {
  my ($data) = @_;

  my @screen_positions;
  my @d016_offsets;

  for my $sinus (@{$data}) {

    my $screen_position = int ($sinus / 0x08);
    my $d016_offset = $sinus % 0x08;

    push @screen_positions, $screen_position;
    push @d016_offsets, $d016_offset;
  }

  return (\@screen_positions, \@d016_offsets);
}
