#!/usr/bin/perl
#
# scripts/extract-gfx-data.pl > target/double-yeah-gfx-data.inc

use utf8;
use bytes;
use strict;
use warnings;

local $| = 1;

use lib q{../../tools/lib};

use Graphics::AdvArtStudio;
use Readonly;
use Source::Data qw(generate_source_data);

Readonly my $SKIP_GFX_DATA_ROWS => 0x05;
Readonly my $FETCH_GFX_DATA_ROWS => 0x19 - 0x08;

my $picture = Graphics::AdvArtStudio->new(file => 'gfx/doubleyeah.aas');

my $bitmap = substr $picture->_get_bitmap_data(), $SKIP_GFX_DATA_ROWS * 0x0140, $FETCH_GFX_DATA_ROWS * 0x0140;
my $screen = substr $picture->_get_screen_data(), $SKIP_GFX_DATA_ROWS * 0x0028, $FETCH_GFX_DATA_ROWS * 0x0028;
my $colors = substr $picture->_get_colors_data(), $SKIP_GFX_DATA_ROWS * 0x0028, $FETCH_GFX_DATA_ROWS * 0x0028;

my @sources = (
  {
    data  => $bitmap,
    label => 'CELTIC_BITMAP',
  },
  {
    data  => $screen,
    label => 'CELTIC_SCREEN',
  },
  {
    data  => $colors,
    label => 'CELTIC_COLORS',
  },
);

for my $source (@sources) {

  my @data = map { ord } split //, $source->{data};

  my $source_code = generate_source_data(
    data                 => \@data,
    name                 => '',
    calculate_hex_values => 1,
    expected_indentation => 16,
    include_comments     => 1,
    values_per_line      => 0x20,
  );

  $source_code =~ m/^(;-+\n$)/m;

  my $label = $source->{label};

  print $+;
  print "                *= $label\n";
  print $source_code;
  print "\n";
}
