/** How to execute this script:
  *
  *   $ export AFTERIMAGE=/home/pkrol/.ivy2/cache/org.c64.attitude/afterimage/jars/afterimage-0.03-SNAPSHOT.jar
  *   $ export IMAGEJ=/home/pkrol/.ivy2/cache/gov.nih.imagej/imagej/jars/imagej-1.46.jar
  *   $ scala -classpath ${AFTERIMAGE}:${IMAGEJ} scripts/convert-png-to-hires-bitmap.scala images/the-end.png images/the-end.hpi
  */

import org.c64.attitude.Afterimage.Colour.Palette
import org.c64.attitude.Afterimage.File.File
import org.c64.attitude.Afterimage.File.Import
import org.c64.attitude.Afterimage.Format.HiResBitmap
import org.c64.attitude.Afterimage.Mode
import org.c64.attitude.Afterimage.View.Image

def usage() {
  println("  Usage: scripts/convert-png-to-hires-bitmap.scala <SOURCE.PNG> <TARGET.HPI>")
}

val source = if (args.length > 0) args(0) else ""
val target = if (args.length > 1) args(1) else ""

if (source == "") {
  println("Source PNG file must be specified")
  usage()
  exit(1)
}

if (!new java.io.File(source).exists) {
  println("Source PNG file does not exist: \"%s\"".format(source))
  usage()
  exit(2)
}

if (target == "") {
  println("Target HiRes bitmap file must be specified")
  usage()
  exit(3)
}

if (new java.io.File(target).exists) {
  println("Target HiRes bitmap file already exists: \"%s\"".format(target))
  usage()
  exit(4)
}

val palette = Palette("default")

println("Importing source image file: \"%s\"".format(source))

val importMode = Import.HiRes(palette)
val picture = File.convert(source, importMode)
val hiResBitmap = HiResBitmap(picture.asInstanceOf[Mode.HiRes])

// Image(picture, palette).show()

println("Writing target image file: \"%s\"".format(target))

hiResBitmap.save(target)
