package Helper;

use utf8;
use bytes;
use strict;
use warnings;

use base qw(Exporter);
our %EXPORT_TAGS = ();
$EXPORT_TAGS{source} = [ qw(&print_data_as_source_code) ];
$EXPORT_TAGS{sinus} = [ qw(&visualize_sinus_comment) ];
$EXPORT_TAGS{all} = [ @{ $EXPORT_TAGS{sinus} }, @{ $EXPORT_TAGS{source} } ];
our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
our @EXPORT = qw();

use Data::Dumper;

sub print_data_as_source_code {
  my ($data, $label, $lo_hi, $last_comment) = @_;

  $label .= "_$lo_hi";
  my $sign = $lo_hi eq 'lo' ? '<' : '>';

  my $indent = length ($label) + 1;

  if ($label) {
    print_source_code_comment($indent, $data);
    print "$label ";
  }

  for (my $i = 0; $i < @{$data}; $i++) {
    my $row = $data->[$i];
    if ($i > 0) {
      print ' ' x $indent;
    }
    print '.db ';
    print join ',', map { sprintf '%s$%04x', $sign, $_ } @{$row};
    print "\n";
  }

  if ($last_comment) {
    print_source_code_comment($indent, $data);
  }

  return;
}

sub print_source_code_comment {
  my ($indent, $data) = @_;

  my $num_row_values = scalar @{$data->[0]};
  my $data_length = $num_row_values * 3 + ($num_row_values - 1);

  my $comment_length = 4 + $data_length + $indent - 1;

  printf ";%s\n", '-' x $comment_length;

  return;
}

sub visualize_sinus_comment {
  my ($data, $max_x, $max_y) = @_;

  my @data = map { $_ + 0 } map { @$_ } @{$data};
  pop @data;

  # Calculate sinus values for all X indices:
  my $ratio = scalar (@data) / $max_x;
  my @sinus;
  for (my $x = 0; $x < $max_x; $x++) {
    my $index = int ($ratio * $x);
    my $value = $data[$index];
    push @sinus, $value;
  }

  my ($max_sinus) = sort { $b <=> $a } @sinus;
  $ratio = $max_y / $max_sinus;

  # Normalize sinus values according to max Y:
  for my $sinus (@sinus) {
    $sinus = int ($ratio * $sinus);
  }

  # Setup printable lines:
  my @lines;
  for (my $y = 0; $y < $max_y; $y++) {
    for (my $x = 0; $x < $max_x; $x++) {
      my $star = $sinus[$x] == $y ? '*' : ' ';
      push @{$lines[$y]}, $star;
    }
  }
  @lines = map { "; $_\n" } map { join '', @$_ } @lines;

  printf ";%s\n", '-' x ($max_x + 1);
  print @lines;

  # printf ";%s\n", '-' x ($max_x + 1);

  return;
}

1;

__END__
