#!/usr/bin/perl
#
# scripts/calculate-raster-downhill.pl

use utf8;
use bytes;
use strict;
use warnings;

use Data::Dumper;
use Readonly;

use FindBin qw($Bin);
use lib $Bin;
use Helper qw(:all);
use List::Util qw(max);
use POSIX;

Readonly my $PI => 3.14;
Readonly my $NUM_VALUES => 250;
Readonly my $MIN_VALUE => 0x0004;
Readonly my $MAX_VALUE => 0x0127;

# Calculate sinus from $MIN_VALUE to $MAX_VALUE:
my $data = calculate_sinus_data();
push @{$data}, 0xffff;

# Print out generated lookup table with lo values:
my @data_split;
while (@{$data}) {
  push @data_split, [ splice @{$data}, 0x00, 0x08 ];
}
visualize_sinus_comment(\@data_split, 58, 24);
print_data_as_source_code(\@data_split, 'raster_downhill_sinus', 'lo', 0);
print_data_as_source_code(\@data_split, 'raster_downhill_sinus', 'hi', 1);

sub calculate_sinus_data {
  my @data;
  for my $index (0x00 .. $NUM_VALUES - 1) {
    my $x = $index / $NUM_VALUES * 9.61;
    my $new_value = (4 * cos ( $x * $PI / 16 ) - sin ( $x * $PI / 2 - $PI / 2 ) + 2);
    push @data, $new_value;
  }
  # values => from 7 to 0 need to be translated into new values => from 4 to 294
  my $value_range = $MAX_VALUE - $MIN_VALUE;
  my $max = max @data;
  @data = map { int ($MAX_VALUE - $_ * $value_range / 7) } @data;
  return \@data;
}
