package D64::Procedures;

use bytes;
use strict;
use warnings;

use base qw(Exporter);
our %EXPORT_TAGS = ();
$EXPORT_TAGS{'all'} = [ qw(&create_disk &write_file_to_disk) ];
our @EXPORT_OK = @{ $EXPORT_TAGS{'all'} };
our @EXPORT = qw();

use Carp qw(croak);
use Data::Dumper;
use D64::Disk::Image qw(:all);

# Create an empty D64 disk image:
sub create_disk {
    my $filename = shift;
    croak q{D64 disk image file name to be created not specified} unless $filename;
    my $name     = shift || 'dj gruby/arsenic';
    my $id       = shift || '12';
    my $d64      = D64::Disk::Image->create_image($filename, D64);
    my $rawname  = $d64->rawname_from_name($d64->ascii_to_petscii($name));
    my $rawid    = $d64->rawname_from_name($d64->ascii_to_petscii($id));
    $d64->format($rawname, $rawid);
    $d64->free_image();
    print qq{\nCreated D64 disk image file: ${filename}\n};
}

# Writes a single file into a given disk image:
sub write_file_to_disk {
    my $disk_image_filename = shift;
    my $file                = shift;
    (my $disk_image_filename_only = $disk_image_filename) =~ s!^.*/!!;
    (my $file_filename_only = $file) =~ s!^.*/!!;
    my $d64 = D64::Disk::Image->load_image($disk_image_filename);
    printf qq{\nWriting file "%s" into "%s" disk image... }, $file_filename_only, $disk_image_filename_only;
    my $size = _write_file_to_disk($d64, $file, $file_filename_only);
    printf qq{wrote %d bytes [ OK ]\n}, $size;
    $d64->free_image();
}

sub _write_file_to_disk {
    my $d64              = shift;
    my $file             = shift;
    my $filename_on_disk = shift;
    croak qq{File "${file}" does not exist} unless -e $file;
    my $rawname = $d64->rawname_from_name($d64->ascii_to_petscii($filename_on_disk));
    my $prg = $d64->open($rawname, T_PRG, F_WRITE);
    open PRG, '<:bytes', $file or croak qq{Couldn't open "${file}" file for reading};
    my $buffer;
    my $filesize = (stat($file))[7];
    sysread PRG, $buffer, $filesize;
    my $size = $prg->write($buffer);
    close PRG;
    $prg->close();
    return $size;
}

1;

__END__
