package Data::Transform;

=head1 NAME

Data::Transform - Miscellaneous data conversion routines

=head1 SYNOPSIS

  use Data::Transform qw(:all);

  # Convert raw binary data to an array of printable hex values:
  my @printable_data = convert_binary_data_to_hex_values(
    data => $binary_data,
  );

=cut

use utf8;
use bytes;
use strict;
use warnings;

use base qw(Exporter);
our %EXPORT_TAGS = ();
$EXPORT_TAGS{binary} = [ qw(&convert_binary_data_to_hex_values) ];
$EXPORT_TAGS{all} = [ @{ $EXPORT_TAGS{binary} } ];
our @EXPORT_OK = @{ $EXPORT_TAGS{all} };
our @EXPORT = qw();

use Data::Dumper;

sub convert_binary_data_to_hex_values {
  my (%args) = @_;
  my $data = $args{data} || [];
  my @data = map { hex unpack 'H*' } split //, $data;
  return @data;
}

1;

__END__