#!/usr/bin/perl
#
# Example:
#
#   $ tools/scripts/write-file-to-d64.pl --filename=<FILENAME.PRG> --image=<DISK.D64>

use bytes;
use strict;
use warnings;

use lib q{tools/lib};

use Carp qw(croak);
use Getopt::Long;

use D64::Procedures qw(write_file_to_disk);

my $disk_image;
my $filename;

my $result = GetOptions(
    q{filename=s} => \$filename,
    q{image=s}    => \$disk_image,
);

help() unless defined $disk_image;
help() unless defined $filename;

croak qq{Program file "${filename}" does not exist} unless -e $filename;
croak qq{Disk image "${disk_image}" does not exist} unless -e $disk_image;

write_file_to_disk($disk_image, $filename);

print qq{\nDone.\n\n};

sub help {
    print <<HELP;

Usage: $0 --filename=<FILENAME.PRG> --image=<DISK.D64>

Writes FILENAME.PRG into DISK.D64 image.

(C) 2012-10-08 DJ Gruby/Arsenic/Oxyron

HELP
    exit 1;
}

1;

__END__